#!/usr/bin/python3

import argparse
import configparser
import json
import os
import shutil

from debian.changelog import Changelog
from debian.deb822 import Deb822

unconfined_apps = ('ubuntu-terminal',
                   'ubuntu-filemanager')

arch_specific_apps = ('terminal',
                      'filemanager',
                      'music',
                      'docviewer')


def find_desktop_files(path=os.path.curdir):
    """Searches path for .desktop files and returns a list of them."""
    desktop_files = []
    for root, dirs, files in os.walk(path):
        desktop_file = filter(lambda x: x.endswith('.desktop'), files)
        for f in desktop_file:
            desktop_files.append(os.path.join(root, f))
    return desktop_files


def get_icon_from_desktop(desktop_file):
    """Obtains icon from desktop file."""
    config = configparser.ConfigParser()
    config.optionxform = str
    config.read(desktop_file)
    return config.get('Desktop Entry', 'Icon')


def fix_exec_inline(exec_line, path):
    exec_line_len = len(exec_line)
    if exec_line_len == 3:
        exec_line = '%s %s %s' % (
            exec_line[0], strip_path(exec_line[1], path), exec_line[2])
    elif exec_line_len == 2:
        exec_line = '%s %s' % (exec_line[0], strip_path(exec_line[1], path))
    elif exec_line_len == 1:
        exec_line = strip_path(exec_line[0], path)
    else:
        raise NotImplementedError('Cannot handle Exec of len %s' %
                                  exec_line_len)
    return exec_line


def add_import(exec_line):
    if 'qmlscene' in exec_line:
        exec_line = '%s -I ./plugins' % exec_line
    return exec_line


def fix_desktop(desktop_file, package_name, path):
    """Sets Exec and Icon to basename path entries."""
    config = configparser.RawConfigParser()
    config.optionxform = str
    config.read(desktop_file)
    icon = config.get('Desktop Entry', 'Icon')
    config.set('Desktop Entry', 'Icon', strip_path(icon, path))
    exec_line = config.get('Desktop Entry', 'Exec').split()
    exec_line = fix_exec_inline(exec_line, path)
    if package_name in arch_specific_apps:
        print('Creating custom exec')
        exec_line = add_import(exec_line)
    config.set('Desktop Entry', 'Exec', exec_line)
    with open(desktop_file, 'w') as f:
        config.write(f, space_around_delimiters=False)


def strip_path(item, path):
    """Returns unstripped path for item."""
    basename = os.path.basename(item)
    if item == basename:
        print('%s is good' % item)
    else:
        for root, sub, files in os.walk(path):
            if basename in files:
                basename = os.path.join(root.lstrip(path + os.path.sep),
                                        basename)
        print('Replacing %s with %s' % (item, basename))
    return basename


def remove_cruft(path=os.path.curdir):
    for cruft_dir in ('debian', 'tests', 'po'):
        cruft_dir = os.path.join(path, cruft_dir)
        if os.path.exists(cruft_dir):
            print('Removing %s' % cruft_dir)
            shutil.rmtree(cruft_dir)


def get_deb_package_dict(path=os.path.curdir):
    changelog_file = os.path.join(path, 'debian', 'changelog')
    control_file = os.path.join(path, 'debian', 'control')
    if not os.path.exists(changelog_file) or not os.path.exists(control_file):
        raise RuntimeError('%s or %s missing' % (changelog_file, control_file))
    changelog = Changelog()
    with open(changelog_file) as f:
        changelog.parse_changelog(f, max_blocks=1)
    entries = {
        'package': str(changelog.package),
        'version': str(changelog.version),
    }
    with open(control_file, 'rt') as f:
        for paragraph in Deb822.iter_paragraphs(f):
            if 'Maintainer' in paragraph:
                entries['maintainer'] = paragraph['Maintainer']
            elif 'Description' in paragraph and \
                    paragraph['Package'] == entries['package']:
                entries['description'] = \
                    paragraph['Description'].split('\n')[0]
    entries['click_name'] = entries['package'].strip('-app')
    return entries


def setup_security_manifest(package_name, path=os.path.curdir):
    security_manifest_file = '%s.json' % package_name
    security_dir = os.path.join(path, 'apparmor')
    security_manifest_path = os.path.join(security_dir,
                                          security_manifest_file)
    if os.path.exists(security_manifest_path):
        print('App armor rules already provided')
        return os.path.join('apparmor', security_manifest_file)
    os.makedirs(security_dir)
    if package_name in unconfined_apps:
        security_part = {'policy_version': 1.0,
                         'template': 'unconfined'
                         }
    else:
        security_part = {'policy_groups': [],
                         'policy_version': 1.0
                         }
    print('Creating security manifest with dictionary', security_part)
    print('Creating', security_manifest_file)
    with open(security_manifest_path, 'w') as f:
        for line in json.dumps(security_part, sort_keys=True, indent=4):
            f.write(line)
    return os.path.join('apparmor', security_manifest_file)


def setup_simple_manifest(data, desktop_file,
                          bzr_source, bzr_revno, path=os.path.curdir):
    manifest_file = os.path.join(path, 'manifest.json')
    if os.path.exists(manifest_file):
        return manifest_file
    package_name = data['package']
    security_manifest_file = setup_security_manifest(package_name, path)
    manifest = {'name': 'com.ubuntu.%s' % package_name,
                'version': data['version'],
                'title': package_name,
                'description': data['description'],
                'maintainer': data['maintainer'],
                'icon': get_icon_from_desktop(desktop_file),
                'framework': 'ubuntu-sdk-13.10',
                'hooks': {package_name: {
                    'apparmor': security_manifest_file,
                    'desktop': os.path.basename(desktop_file),
                    },
                },
                'x-source': {
                    'vcs-bzr': bzr_source,
                    'vcs-bzr-revno': bzr_revno
                    },
                }
    if package_name in arch_specific_apps:
        manifest['architecture'] = 'armhf'
    print('Creating manifest with dictionary', manifest)
    with open(manifest_file, 'w') as f:
        for line in json.dumps(manifest, sort_keys=True, indent=4):
            f.write(line)
    return None


def add_source_info(manifest_file, bzr_source, bzr_revno):
    with open(manifest_file) as f:
        manifest = json.loads(f.read())
    manifest['x-source'] = {
        'vcs-bzr': bzr_source,
        'vcs-bzr-revno': bzr_revno
        }
    with open(manifest_file, 'w') as f:
        for line in json.dumps(manifest, sort_keys=True, indent=4):
            f.write(line)


def get_parser():
    parser = argparse.ArgumentParser(
        description='''Parses sources and stages them to build click
                       packages out of them.''')
    parser.add_argument('--source-directory', dest='path',
                        default=os.path.curdir)
    parser.add_argument('--bzr-source')
    parser.add_argument('--bzr-revno')
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    if not os.path.exists(args.path):
        raise RuntimeError('Directory %s does not exist' % args.path)
    entries = get_deb_package_dict(args.path)
    entries['package'] = entries['package'].strip('-app')
    desktop_files = find_desktop_files(args.path)
    for desktop_file in desktop_files:
        print('Fixing desktop file %s' % desktop_file)
        fix_desktop(desktop_file, entries['package'], args.path)

    manifest_file = setup_simple_manifest(
        data=entries,
        desktop_file=desktop_files[0],
        bzr_source=args.bzr_source,
        bzr_revno=args.bzr_revno,
        path=args.path)
    if manifest_file:
        add_source_info(manifest_file, args.bzr_source, args.bzr_revno)
    remove_cruft(args.path)
